import 'package:flutter/material.dart';
import 'package:intl/intl.dart';
import 'package:localization/localization.dart';
import 'package:loris/partials/main_scaffold.dart';
import 'pages/login.dart';
import 'business_logic/settings.dart' as settings;
import 'package:flutter_localizations/flutter_localizations.dart';
import 'themes/themes.dart' as themes;
import 'global.dart' as global;
import 'business_logic/auth/oauth.dart' as oauth;

String _initRoute = "/";
ThemeData theme = themes.getTheme(themes.available[0]);
Locale activeLocale = const Locale("en");

void main() async {
  Intl.defaultLocale = "en";
  global.settings = await settings.Settings.create();
  activeLocale = global.settings!.locale;

  // check if all information is available
  if (global.settings!.authCode == "") {
    _initRoute = "/login";
  } else {
    await oauth.refreshToken();
  }
  runApp(const Loris());
}

class Loris extends StatefulWidget {
  const Loris({Key? key}) : super(key: key);

  @override
  State<Loris> createState() => _LorisState();
}

class _LorisState extends State<Loris> {
  @override
  Widget build(BuildContext context) {
    LocalJsonLocalization.delegate.directories = ['lib/i18n'];
    return MaterialApp(
      theme: theme,
      locale: activeLocale,
      supportedLocales: global.availableLocales,
      localizationsDelegates: [
        GlobalCupertinoLocalizations.delegate,
        GlobalWidgetsLocalizations.delegate,
        GlobalMaterialLocalizations.delegate,
        LocalJsonLocalization.delegate,
      ],
      initialRoute: _initRoute,
      routes: {
        '/': (context) => const MainScaffold(),
        '/login': (context) => const Login(),
      },
    );
  }
}
